# -*- coding: utf-8 -*-

r"""
A Local-best Particle Swarm Optimization (lbest PSO) algorithm.

Similar to global-best PSO, it takes a set of candidate solutions,
and finds the best solution using a position-velocity update method.
However, it uses a ring topology, thus making the particles
attracted to its corresponding neighborhood.

The position update can be defined as:

.. math::

   x_{i}(t+1) = x_{i}(t) + v_{i}(t+1)

Where the position at the current timestep :math:`t` is updated using
the computed velocity at :math:`t+1`. Furthermore, the velocity update
is defined as:

.. math::

   v_{ij}(t + 1) = m * v_{ij}(t) + c_{1}r_{1j}(t)[y_{ij}(t) − x_{ij}(t)] + c_{2}r_{2j}(t)[\hat{y}_{j}(t) − x_{ij}(t)]

However, in local-best PSO, a particle doesn't compare itself to the
overall performance of the swarm. Instead, it looks at the performance
of its nearest-neighbours, and compares itself with them. In general,
this kind of topology takes much more time to converge, but has a more
powerful explorative feature.

In this implementation, a neighbor is selected via a k-D tree
imported from :code:`scipy`. Distance are computed with either
the L1 or L2 distance. The nearest-neighbours are then queried from
this k-D tree.

An example usage is as follows:

.. code-block:: python

    import pyswarms as ps
    from pyswarms.utils.functions import single_obj as fx

    # Set-up hyperparameters
    options = {'c1': 0.5, 'c2': 0.3, 'w': 0.9, 'k': 3, 'p': 2}

    # Call instance of LBestPSO with a neighbour-size of 3 determined by
    # the L2 (p=2) distance.
    optimizer = ps.single.LocalBestPSO(n_particles=10, dimensions=2,
                                       options=options)

    # Perform optimization
    stats = optimizer.optimize(fx.sphere_func, iters=100)

This algorithm was adapted from one of the earlier works of
J. Kennedy and R.C. Eberhart in Particle Swarm Optimization
[IJCNN1995]_ [MHS1995]_

.. [IJCNN1995] J. Kennedy and R.C. Eberhart, "Particle Swarm Optimization,"
    Proceedings of the IEEE International Joint Conference on Neural
    Networks, 1995, pp. 1942-1948.

.. [MHS1995] J. Kennedy and R.C. Eberhart, "A New Optimizer using Particle
    Swarm Theory,"  in Proceedings of the Sixth International
    Symposium on Micromachine and Human Science, 1995, pp. 39–43.
"""

# Import from stdlib
import logging

# Import modules
import numpy as np

# Import from package
from ..base import SwarmOptimizer
from ..backend.operators import compute_pbest
from ..backend.topology import Ring
from ..utils.console_utils import cli_print, end_report


class LocalBestPSO(SwarmOptimizer):

    def assertions(self):
        """Assertion method to check various inputs.

        Raises
        ------
        KeyError
            When one of the required dictionary keys is missing.
        ValueError
            When the number of neighbors is not within the range :code:`[0, n_particles]`.
            When the p-value is not in the list of values :code:`[1,2]`.
        """
        super(LocalBestPSO, self).assertions()

        if not all(key in self.options for key in ('k', 'p')):
            raise KeyError('Missing either k or p in options')
        if not 0 <= self.k <= self.n_particles:
            raise ValueError('No. of neighbors must be between 0 and no. '
                             'of particles.')
        if self.p not in [1, 2]:
            raise ValueError('p-value should either be 1 (for L1/Minkowski) '
                             'or 2 (for L2/Euclidean).')

    def __init__(self, n_particles, dimensions, options, bounds=None,
                 velocity_clamp=None, center=1.00, ftol=-np.inf, init_pos=None):
        """Initializes the swarm.

        Attributes
        ----------
        n_particles : int
            number of particles in the swarm.
        dimensions : int
            number of dimensions in the space.
        bounds : tuple of np.ndarray, optional (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum bound
            while the second entry is the maximum bound. Each array must
            be of shape :code:`(dimensions,)`.
        velocity_clamp : tuple (default is :code:`(0,1)`)
            a tuple of size 2 where the first entry is the minimum velocity
            and the second entry is the maximum velocity. It
            sets the limits for velocity clamping.
        center : list (default is :code:`None`)
            an array of size :code:`dimensions`
        ftol : float
            relative error in objective_func(best_pos) acceptable for
            convergence
        options : dict with keys :code:`{'c1', 'c2', 'w', 'k', 'p'}`
            a dictionary containing the parameters for the specific
            optimization technique
                * c1 : float
                    cognitive parameter
                * c2 : float
                    social parameter
                * w : float
                    inertia parameter
                * k : int
                    number of neighbors to be considered. Must be a
                    positive integer less than :code:`n_particles`
                * p: int {1,2}
                    the Minkowski p-norm to use. 1 is the
                    sum-of-absolute values (or L1 distance) while 2 is
                    the Euclidean (or L2) distance.
        """
        # Initialize logger
        self.logger = logging.getLogger(__name__)
        # Assign k-neighbors and p-value as attributes
        self.k, self.p = options['k'], options['p']
        # Initialize parent class
        super(LocalBestPSO, self).__init__(n_particles=n_particles, dimensions=dimensions,
                                           options=options, bounds=bounds,
                                           velocity_clamp=velocity_clamp,
                                           center=center, ftol=ftol,
        init_pos=init_pos)
        # Invoke assertions
        self.assertions()
        # Initialize the resettable attributes
        self.reset()
        # Initialize the topology
        self.top = Ring()

    def optimize(self, objective_func, iters, print_step=1, verbose=1):
        """Optimizes the swarm for a number of iterations.

        Performs the optimization to evaluate the objective
        function :code:`f` for a number of iterations :code:`iter.`

        Parameters
        ----------
        objective_func : function
            objective function to be evaluated
        iters : int
            number of iterations
        print_step : int (default is 1)
            amount of steps for printing into console.
        verbose : int  (default is 1)
            verbosity setting.

        Returns
        -------
        tuple
            the local best cost and the local best position among the
            swarm.
        """
        for i in range(iters):
            # Compute cost for current position and personal best
            self.swarm.current_cost = objective_func(self.swarm.position)
            self.swarm.pbest_cost = objective_func(self.swarm.pbest_pos)
            self.swarm.pbest_pos, self.swarm.pbest_cost = compute_pbest(self.swarm)
            best_cost_yet_found = np.min(self.swarm.best_cost)
            # Update gbest from neighborhood
            self.swarm.best_pos, self.swarm.best_cost = self.top.compute_gbest(self.swarm,
                                                                               self.p,
                                                                               self.k)
            # Print to console
            if i % print_step == 0:
                cli_print('Iteration %s/%s, cost: %s' %
                          (i+1, iters, np.min(self.swarm.best_cost)), verbose, 2,
                          logger=self.logger)
            # Save to history
            hist = self.ToHistory(best_cost=self.swarm.best_cost,
                                  mean_pbest_cost=np.mean(self.swarm.pbest_cost),
                                  mean_neighbor_cost=np.mean(self.swarm.best_cost),
                                  position=self.swarm.position,
                                  velocity=self.swarm.velocity)
            self._populate_history(hist)
            # Verify stop criteria based on the relative acceptable cost ftol
            relative_measure = self.ftol*(1 + np.abs(best_cost_yet_found))
            if np.abs(self.swarm.best_cost - best_cost_yet_found) < relative_measure:
                break
            # Perform position velocity update
            self.swarm.velocity = self.top.compute_velocity(self.swarm, self.velocity_clamp)
            self.swarm.position = self.top.compute_position(self.swarm, self.bounds)
        # Obtain the final best_cost and the final best_position
        final_best_cost = self.swarm.best_cost.copy()
        final_best_pos = self.swarm.best_pos.copy()
        end_report(final_best_cost, final_best_pos, verbose, logger=self.logger)
        return (final_best_cost, final_best_pos)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import pytest
import numpy as np
from collections import namedtuple

# Import from package
from pyswarms.utils.functions import single_obj as fx

def test_sphere_output(common_minima):
    """Tests sphere function output."""
    assert np.array_equal(fx.sphere_func(common_minima), np.zeros((3,)))

def test_rastrigin_output(common_minima):
    """Tests rastrigin function output."""
    assert np.array_equal(fx.rastrigin_func(common_minima), np.zeros(3))

def test_ackley_output(common_minima):
    """Tests ackley function output."""
    assert np.isclose(fx.ackley_func(common_minima), np.zeros(3)).all()

def test_rosenbrock_output(common_minima2):
    """Tests rosenbrock function output."""
    assert np.array_equal(fx.rosenbrock_func(common_minima2).all(),np.zeros(3).all())

def test_beale_output(common_minima2):
    """Tests beale function output."""
    assert np.isclose(fx.beale_func([3, 0.5] * common_minima2), np.zeros(3)).all()

def test_goldstein_output(common_minima2):
    """Tests goldstein-price function output."""
    assert np.isclose(fx.goldstein_func([0, -1] * common_minima2), (3 * np.ones(3))).all()

def test_booth_output(common_minima2):
    """Test booth function output."""
    assert np.isclose(fx.booth_func([1, 3] * common_minima2), np.zeros(3)).all()

def test_bukin6_output(common_minima2):
    """Test bukin function output."""
    assert np.isclose(fx.bukin6_func([-10, 1] * common_minima2), np.zeros(3)).all()

def test_bukin6_output(common_minima):
    """Test bukin function output."""
    assert np.isclose(fx.matyas_func(common_minima), np.zeros(3)).all()

def test_levi_output(common_minima2):
    """Test levi function output."""
    assert np.isclose(fx.levi_func(common_minima2), np.zeros(3)).all()

def test_schaffer2_output(common_minima):
    """Test schaffer2 function output."""
    assert np.isclose(fx.schaffer2_func(common_minima), np.zeros(3)).all()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import pytest
import numpy as np

# Import from package
from pyswarms.backend.topology import Star


def test_compute_gbest_return_values(swarm):
    """Test if compute_gbest() gives the expected return values"""
    topology = Star()
    expected_cost = 1
    expected_pos = np.array([1,2,3])
    pos, cost = topology.compute_gbest(swarm)
    assert cost == expected_cost
    assert (pos == expected_pos).all()

@pytest.mark.parametrize('clamp', [None, (0,1), (-1,1)])
def test_compute_velocity_return_values(swarm, clamp):
    """Test if compute_velocity() gives the expected shape and range"""
    topology = Star()
    v = topology.compute_velocity(swarm, clamp)
    assert v.shape == swarm.position.shape
    if clamp is not None:
        assert (clamp[0] <= v).all() and (clamp[1] >= v).all()

@pytest.mark.parametrize('bounds', [None,  ([-5,-5,-5],[5,5,5]),
    ([-10, -10, -10],[10, 10, 10])])
def test_compute_position_return_values(swarm, bounds):
    """Test if compute_position() gives the expected shape and range"""
    topology = Star()
    p = topology.compute_position(swarm, bounds)
    assert p.shape == swarm.velocity.shape
    if bounds is not None:
        assert (bounds[0] <= p).all() and (bounds[1] >= p).all()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Fixtures for tests"""

# Import modules
import os
import pytest
import numpy as np
from mock import Mock
import matplotlib as mpl

if os.environ.get('DISPLAY','') == '':
    mpl.use('Agg')

# Import from package
from pyswarms.single import GlobalBestPSO
from pyswarms.utils.environments import PlotEnvironment
from pyswarms.utils.functions.single_obj import sphere_func

@pytest.fixture
def mock_pso():
    """Returns a function that mocks a PSO class with missing attributes"""
    def _mock_pso(index):
        class_methods = [
            'get_cost_history',
            'get_pos_history',
            'get_velocity_history',
            'optimize',
            'reset'
        ]
        get_specs = lambda idx: [x for i,x in enumerate(class_methods) if i != idx]
        return Mock(spec=get_specs(index))
    return _mock_pso

@pytest.fixture
def plot_environment():
    """Returns a PlotEnvironment instance"""
    optimizer = GlobalBestPSO(10, 3, options={'c1': 0.5, 'c2': 0.3, 'w': 0.9})
    return PlotEnvironment(optimizer, sphere_func, 1000)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import pytest
import numpy as np

# Import from package
from pyswarms.discrete import BinaryPSO
from pyswarms.utils.functions.single_obj import sphere_func

@pytest.mark.parametrize('options', [
    {'c2':0.7, 'w':0.5, 'k': 2, 'p': 2},
    {'c1':0.5, 'w':0.5, 'k': 2, 'p': 2},
    {'c1':0.5, 'c2':0.7, 'k': 2, 'p': 2},
    {'c1':0.5, 'c2':0.7, 'w':0.5, 'p': 2},
    {'c1':0.5, 'c2':0.7, 'w':0.5, 'k': 2}
])
def test_keyword_exception(options):
    """Tests if exceptions are thrown when keywords are missing"""
    with pytest.raises(KeyError):
        BinaryPSO(5, 2, options)

@pytest.mark.parametrize('options', [
    {'c1':0.5, 'c2':0.7, 'w':0.5, 'k':-1, 'p':2},
    {'c1':0.5, 'c2':0.7, 'w':0.5, 'k':6, 'p':2},
    {'c1':0.5, 'c2':0.7, 'w':0.5, 'k':2, 'p':5}
])
def test_invalid_k_or_p_values(options):
    """Tests if exception is thrown when passing
    an invalid value for k or p"""
    with pytest.raises(ValueError):
        BinaryPSO(5, 2, options)

@pytest.mark.parametrize('velocity_clamp', [[1, 3],np.array([1, 3])])
def test_vclamp_type_exception(velocity_clamp, options):
    """Tests if exception is raised when velocity_clamp type is not a
    tuple"""
    with pytest.raises(TypeError):
        BinaryPSO(5, 2, velocity_clamp=velocity_clamp, options=options)

@pytest.mark.parametrize('velocity_clamp', [(1,1,1), (2,3,1)])
def test_vclamp_shape_exception(velocity_clamp, options):
    """Tests if exception is raised when velocity_clamp's size is not equal
    to 2"""
    with pytest.raises(IndexError):
        BinaryPSO(5, 2, velocity_clamp=velocity_clamp, options=options)

@pytest.mark.parametrize('velocity_clamp', [(3,2),(10,8)])
def test_vclamp_maxmin_exception(velocity_clamp, options):
    """Tests if the max velocity_clamp is less than min velocity_clamp and
    vice-versa"""
    with pytest.raises(ValueError):
        BinaryPSO(5, 2, velocity_clamp=velocity_clamp, options=options)

def test_reset_default_values(binary_reset):
    """Tests if best cost and best pos are set properly when the reset()
    method is called"""
    assert binary_reset.swarm.best_cost == np.inf
    assert set(binary_reset.swarm.best_pos) == set(np.array([]))

def test_training_history_shape(binary_history):
    """Test if training histories are of expected shape"""
    assert binary_history.get_cost_history.shape == (1000,)
    assert binary_history.get_mean_pbest_history.shape == (1000,)
    assert binary_history.get_mean_neighbor_history.shape == (1000,)
    assert binary_history.get_pos_history.shape == (1000, 10, 2)
    assert binary_history.get_velocity_history.shape == (1000, 10, 2)



"""
The :code:`pyswarms.backend.topology` contains various topologies that dictate
particle behavior. These topologies implement three methods:
    - compute_best_particle(): gets the position and cost of the best particle   in the swarm
    - update_velocity(): updates the velocity-matrix depending on the topology.
    - update_position(): updates the position-matrix depending on the topology.
"""

from .star import Star
from .ring import Ring


__all__ = [
    "Star",
    "Ring"
]


# -*- coding: utf-8 -*-

"""
Swarm Generation Backend

This module abstracts how a swarm is generated. You can see its
implementation in our base classes. In addition, you can use all the methods
here to dictate how a swarm is initialized for your custom PSO.

"""

# Import modules
import numpy as np

# Import from package
from .swarms import Swarm


def generate_swarm(n_particles, dimensions, bounds=None, center=1.00, init_pos=None):
    """Generates a swarm

    Parameters
    ----------
    n_particles : int
        number of particles to be generated in the swarm.
    dimensions: int
        number of dimensions to be generated in the swarm
    bounds : tuple of :code:`np.ndarray` or list (default is :code:`None`)
        a tuple of size 2 where the first entry is the minimum bound while
        the second entry is the maximum bound. Each array must be of shape
        :code:`(dimensions,)`.
    center : :code:`numpy.ndarray` or :code:`float` (default is :code:`1`)
        controls the mean or center whenever the swarm is generated randomly.
    init_pos : :code:`numpy.ndarray` (default is :code:`None`)
        option to explicitly set the particles' initial positions. Set to
        :code:`None` if you wish to generate the particles randomly.

    Returns
    -------
    numpy.ndarray
        swarm matrix of shape (n_particles, n_dimensions)
    """
    try:
        if init_pos is not None:
            # There is user-defined initial position
            if bounds is None:
                pos = init_pos
            else:
                if not (np.all(bounds[0] <= init_pos) and np.all(init_pos <= bounds[1])):
                    raise ValueError('User-defined init_pos is out of bounds.')
                pos = init_pos
        else:
            # There is no user-defined initial position
            if bounds is None:
                pos = center * np.random.uniform(low=0.0, high=1.0, size=(n_particles, dimensions))
            else:
                lb, ub = bounds
                min_bounds = np.repeat(np.array(lb)[np.newaxis, :], n_particles, axis=0)
                max_bounds = np.repeat(np.array(ub)[np.newaxis, :], n_particles, axis=0)
                pos = center * np.random.uniform(low=min_bounds, high=max_bounds,
                                                size=(n_particles, dimensions))
    except ValueError:
        raise
    else:
        return pos

def generate_discrete_swarm(n_particles, dimensions, binary=False, init_pos=None):
    """Generates a discrete swarm
    
    Parameters
    ----------
    n_particles : int
        number of particles to be generated in the swarm.
    dimensions: int
        number of dimensions to be generated in the swarm.
    binary : bool (default is :code:`False`)
        generate a binary matrix
    init_pos : :code:`numpy.ndarray` (default is :code:`None`)
        option to explicitly set the particles' initial positions. Set to
        :code:`None` if you wish to generate the particles randomly.
    """
    try:
        if init_pos is not None:
            # There is user-defined initial position
            if binary:
                # Check if the initialized position is binary
                if not len(np.unique(init_pos)) == 2:
                    raise ValueError('User-defined init_pos is not binary!')
                pos = init_pos
            else:
                pos = init_pos
        else:
            # There is no user-defined initial position
            if binary:
                pos = np.random.randint(2, size=(n_particles, dimensions))
            else:
                pos = np.random.random_sample(size=(n_particles, dimensions)).argsort(axis=1)
    except ValueError:
        raise
    else:
        return pos

def generate_velocity(n_particles, dimensions, clamp=None):
    """Initializes a velocity vector

    Parameters
    ----------
    n_particles : int
        number of particles to be generated in the swarm.
    dimensions: int
        number of dimensions to be generated in the swarm.
    clamp : tuple of floats (default is :code:`None`)
        a tuple of size 2 where the first entry is the minimum velocity
        and the second entry is the maximum velocity. It
        sets the limits for velocity clamping.

    Returns
    -------
    numpy.ndarray
        velocity matrix of shape (n_particles, dimensions)
    """
    try:
        min_velocity, max_velocity = (0,1) if clamp==None else clamp
        velocity = ((max_velocity - min_velocity) 
                 * np.random.random_sample(size=(n_particles, dimensions)) 
                 + min_velocity)
    except (ValueError, TypeError):
        raise
    else:
        return velocity

def create_swarm(n_particles, dimensions, discrete=False, binary=False,
                options={}, bounds=None, center=1.0, init_pos=None, clamp=None):
    """Abstracts the generate_swarm() and generate_velocity() methods
    
    Parameters
    ----------
    n_particles : int
        number of particles to be generated in the swarm.
    dimensions: int
        number of dimensions to be generated in the swarm
    options : dict (default is empty dict :code:`{}`)
        Swarm options, for example, c1, c2, etc.
    discrete : bool (default is :code:`False`)
        Creates a discrete swarm
    binary : bool (default is :code:`False`)
        generate a binary matrix
    bounds : tuple of :code:`np.ndarray` or list (default is :code:`None`)
        a tuple of size 2 where the first entry is the minimum bound while
        the second entry is the maximum bound. Each array must be of shape
        :code:`(dimensions,)`.
    center : :code:`numpy.ndarray` (default is :code:`1`)
        a list of initial positions for generating the swarm
    init_pos : :code:`numpy.ndarray` (default is :code:`None`)
        option to explicitly set the particles' initial positions. Set to
        :code:`None` if you wish to generate the particles randomly.
    clamp : tuple of floats (default is :code:`None`)
        a tuple of size 2 where the first entry is the minimum velocity
        and the second entry is the maximum velocity. It
        sets the limits for velocity clamping.

    Returns
    -------
    pyswarms.backend.swarms.Swarm
        a Swarm class
    """
    if discrete:
        position = generate_discrete_swarm(n_particles, dimensions, binary=binary)
    else:
        position = generate_swarm(n_particles, dimensions, bounds=bounds,
                                  center=center, init_pos=init_pos)

    velocity = generate_velocity(n_particles, dimensions, clamp=clamp)
    return Swarm(position, velocity, options=options)


# -*- coding: utf-8 -*-

"""
Swarm Class Backend

This module implements a Swarm class that holds various attributes in
the swarm such as position, velocity, options, etc. You can use this
as input to most backend cases
"""

# Import modules
import numpy as np
from attr import (attrs, attrib)
from attr.validators import instance_of

@attrs
class Swarm(object):
    """A Swarm Class
    
    This class offers a generic swarm that can be used in most use-cases
    such as single-objective optimization, etc. It contains various attributes
    that are commonly-used in most swarm implementations.

    To initialize this class, **simply supply values for the position and
    velocity matrix**. The other attributes are automatically filled. If you want to
    initialize random values, take a look at: 

    * :func:`pyswarms.backend.generators.generate_swarm`: for generating positions randomly.
    * :func:`pyswarms.backend.generators.generate_velocity`: for generating velocities randomly. 

    If your swarm requires additional parameters (say c1, c2, and w in gbest
    PSO), simply pass them to the :code:`options` dictionary.

    As an example, say we want to create a swarm by generating particles
    randomly. We can use the helper methods above to do our job:

    .. code-block:: python

        import pyswarms.backend as P
        from pyswarms.backend.swarms import Swarm

        # Let's generate a 10-particle swarm with 10 dimensions
        init_positions = P.generate_swarm(n_particles=10, dimensions=10)
        init_velocities = P.generate_velocity(n_particles=10, dimensions=10)
        # Say, particle behavior is governed by parameters `foo` and `bar`
        my_options = {'foo': 0.4, 'bar': 0.6}
        # Initialize the swarm
        my_swarm = Swarm(position=init_positions, velocity=init_velocities, options=my_options)

    From there, you can now use all the methods in :mod:`pyswarms.backend`.
    Of course, the process above has been abstracted by the method
    :func:`pyswarms.backend.generators.create_swarm` so you don't have to
    write the whole thing down.


    Attributes
    ----------
    position : numpy.ndarray
        position-matrix at a given timestep of shape :code:`(n_particles, dimensions)`
    velocity : numpy.ndarray
        velocity-matrix at a given timestep of shape :code:`(n_particles, dimensions)`
    n_particles : int (default is :code:`position.shape[0]`)
        number of particles in a swarm.
    dimensions : int (default is :code:`position.shape[1]`)
        number of dimensions in a swarm.
    options : dict (default is empty dictionary)
        various options that govern a swarm's behavior.
    pbest_pos : numpy.ndarray (default is :code:`None`)
        personal best positions of each particle of shape :code:`(n_particles, dimensions)`
    best_pos : numpy.ndarray (default is empty array)
        best position found by the swarm of shape :code:`(dimensions, )`
    pbest_cost : numpy.ndarray (default is empty array)
        personal best costs of each particle of shape :code:`(n_particles, )`
    best_cost : float (default is :code:`np.inf`)
        best cost found by the swarm 
    current_cost : numpy.ndarray (default is empty array)
        the current cost found by the swarm of shape :code:`(n_particles, dimensions)`
    """
    # Required attributes
    position = attrib(type=np.ndarray, validator=instance_of(np.ndarray))
    velocity = attrib(type=np.ndarray, validator=instance_of(np.ndarray))
    # With defaults
    n_particles = attrib(type=int, validator=instance_of(int))
    dimensions = attrib(type=int, validator=instance_of(int))
    options = attrib(type=dict, default={}, validator=instance_of(dict))
    pbest_pos = attrib(type=np.ndarray, validator=instance_of(np.ndarray))
    best_pos  = attrib(type=np.ndarray, default=np.array([]), validator=instance_of(np.ndarray))
    pbest_cost = attrib(type=np.ndarray, default=np.array([]), validator=instance_of(np.ndarray))
    best_cost = attrib(type=float, default=np.inf, validator=instance_of((int, float)))
    current_cost = attrib(type=np.ndarray, default=np.array([]), validator=instance_of(np.ndarray))

    @n_particles.default
    def n_particles_default(self):
        return self.position.shape[0]

    @dimensions.default
    def dimensions_default(self):
        return self.position.shape[1]

    @pbest_pos.default
    def pbest_pos_default(self):
        return self.position


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `pyswarms` package."""

# Import modules
import pytest
import numpy as np
from collections import namedtuple

# Import from package
from pyswarms.utils.functions import single_obj as fx

def test_beale_dim_fail(outdim):
    """Test beale dim exception"""
    with pytest.raises(IndexError):
        fx.beale_func(outdim)

def test_goldstein_dim_fail(outdim):
    """Test goldstein dim exception"""
    with pytest.raises(IndexError):
        fx.goldstein_func(outdim)

def test_booth_dim_fail(outdim):
    """Test booth dim exception"""
    with pytest.raises(IndexError):
        fx.booth_func(outdim)

def test_bukin6_dim_fail(outdim):
    """Test bukin6 dim exception"""
    with pytest.raises(IndexError):
        fx.bukin6_func(outdim)

def test_matyas_dim_fail(outdim):
    """Test matyas dim exception"""
    with pytest.raises(IndexError):
        fx.matyas_func(outdim)

def test_levi_dim_fail(outdim):
    """Test levi dim exception"""
    with pytest.raises(IndexError):
        fx.levi_func(outdim)

def test_schaffer2_dim_fail(outdim):
    """Test schaffer2 dim exception"""
    with pytest.raises(IndexError):
        fx.schaffer2_func(outdim)


"""
The mod:`pyswarms.utils.environments` module implements various
optimization environments to analyze optimizer performance or search
better parameters
"""

from .plot_environment import PlotEnvironment

__all__ = [
    "PlotEnvironment"
    ]



# -*- coding: utf-8 -*-

r"""
A Binary Particle Swarm Optimization (binary PSO) algorithm.

It takes a set of candidate solutions, and tries to find the best
solution using a position-velocity update method. Unlike
:mod:`pyswarms.single.gb` and :mod:`pyswarms.single.lb`, this technique
is often applied to discrete binary problems such as job-shop scheduling,
sequencing, and the like.

The update rule for the velocity is still similar, as shown in the
proceeding equation:

.. math::

   v_{ij}(t + 1) = m * v_{ij}(t) + c_{1}r_{1j}(t)[y_{ij}(t) − x_{ij}(t)] + c_{2}r_{2j}(t)[\hat{y}_{j}(t) − x_{ij}(t)]

For the velocity update rule, a particle compares its current position
with respect to its neighbours. The nearest neighbours are being
determined by a kD-tree given a distance metric, similar to local-best
PSO. However, this whole behavior can be modified into a global-best PSO
by changing the nearest neighbours equal to the number of particles in
the swarm. In this case, all particles see each other, and thus a global
best particle can be established.

In addition, one notable change for binary PSO is that the position
update rule is now decided upon by the following case expression:

.. math::

   X_{ij}(t+1) = \left\{\begin{array}{lr}
        0, & \text{if } \text{rand() } \geq S(v_{ij}(t+1))\\
        1, & \text{if } \text{rand() } < S(v_{ij}(t+1))
        \end{array}\right\}

Where the function :math:`S(x)` is the sigmoid function defined as:

.. math::

   S(x) = \dfrac{1}{1 + e^{-x}}

This enables the algorithm to output binary positions rather than
a stream of continuous values as seen in global-best or local-best PSO.

This algorithm was adapted from the standard Binary PSO work of J. Kennedy and
R.C. Eberhart in Particle Swarm Optimization [SMC1997]_.

.. [SMC1997] J. Kennedy and R.C. Eberhart, "A discrete binary version of
    particle swarm algorithm," Proceedings of the IEEE International
    Conference on Systems, Man, and Cybernetics, 1997.
"""

# Import from stdlib
import logging

# Import modules
import numpy as np

# Import from package
from ..base import DiscreteSwarmOptimizer
from ..backend.operators import compute_pbest
from ..backend.topology import Ring
from ..utils.console_utils import cli_print, end_report

class BinaryPSO(DiscreteSwarmOptimizer):

    def assertions(self):
        """Assertion method to check various inputs.

        Raises
        ------
        KeyError
            When one of the required dictionary keys is missing.
        ValueError
            When the number of neighbors is not within the range :code:`[0, n_particles]`.
            When the p-value is not in the list of values :code:`[1,2]`.
        """
        super(BinaryPSO, self).assertions()

        if not all(key in self.options for key in ('k', 'p')):
            raise KeyError('Missing either k or p in options')
        if not 0 <= self.k <= self.n_particles:
            raise ValueError('No. of neighbors must be between 0 and no. of'
                             'particles.')
        if self.p not in [1, 2]:
            raise ValueError('p-value should either be 1 (for L1/Minkowski)'
                             'or 2 (for L2/Euclidean).')

    def __init__(self, n_particles, dimensions, options, init_pos=None,
                 velocity_clamp=None, ftol=-np.inf):
        """Initializes the swarm.

        Attributes
        ----------
        n_particles : int
            number of particles in the swarm.
        dimensions : int
            number of dimensions in the space.
        velocity_clamp : tuple (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum velocity
            and the second entry is the maximum velocity. It
            sets the limits for velocity clamping.
        options : dict with keys :code:`{'c1', 'c2', 'k', 'p'}`
            a dictionary containing the parameters for the specific
            optimization technique
                * c1 : float
                    cognitive parameter
                * c2 : float
                    social parameter
                * w : float
                    inertia parameter
                * k : int
                    number of neighbors to be considered. Must be a
                    positive integer less than :code:`n_particles`
                * p: int {1,2}
                    the Minkowski p-norm to use. 1 is the
                    sum-of-absolute values (or L1 distance) while 2 is
                    the Euclidean (or L2) distance.
        """
        # Initialize logger
        self.logger = logging.getLogger(__name__)
        # Assign k-neighbors and p-value as attributes
        self.k, self.p = options['k'], options['p']
        # Initialize parent class
        super(BinaryPSO, self).__init__(n_particles=n_particles, dimensions=dimensions,
                                        binary=True, options=options, init_pos=init_pos,
                                        velocity_clamp=velocity_clamp, ftol=ftol)
        # Invoke assertions
        self.assertions()
        # Initialize the resettable attributes
        self.reset()
        # Initialize the topology
        self.top = Ring()

    def optimize(self, objective_func, iters, print_step=1, verbose=1):
        """Optimizes the swarm for a number of iterations.

        Performs the optimization to evaluate the objective
        function :code:`f` for a number of iterations :code:`iter.`

        Parameters
        ----------
        objective_func : function
            objective function to be evaluated
        iters : int
            number of iterations
        print_step : int (the default is 1)
            amount of steps for printing into console.
        verbose : int  (the default is 1)
            verbosity setting.

        Returns
        -------
        tuple
            the local best cost and the local best position among the
            swarm.
        """
        for i in range(iters):
            # Compute cost for current position and personal best
            self.swarm.current_cost = objective_func(self.swarm.position)
            self.swarm.pbest_cost = objective_func(self.swarm.pbest_pos)
            self.swarm.pbest_pos, self.swarm.pbest_cost = compute_pbest(self.swarm)
            best_cost_yet_found = np.min(self.swarm.best_cost)
            # Update gbest from neighborhood
            self.swarm.best_pos, self.swarm.best_cost = self.top.compute_gbest(self.swarm,
                                                                               self.p,
                                                                               self.k)
            # Print to console
            if i % print_step == 0:
                cli_print('Iteration %s/%s, cost: %s' %
                          (i+1, iters, np.min(self.swarm.best_cost)), verbose, 2,
                          logger=self.logger)
            # Save to history
            hist = self.ToHistory(best_cost=self.swarm.best_cost,
                                  mean_pbest_cost=np.mean(self.swarm.pbest_cost),
                                  mean_neighbor_cost=np.mean(self.swarm.best_cost),
                                  position=self.swarm.position,
                                  velocity=self.swarm.velocity)
            self._populate_history(hist)
            # Verify stop criteria based on the relative acceptable cost ftol
            relative_measure = self.ftol*(1 + np.abs(best_cost_yet_found))
            if np.abs(self.swarm.best_cost - best_cost_yet_found) < relative_measure:
                break
            # Perform position velocity update
            self.swarm.velocity = self.top.compute_velocity(self.swarm, self.velocity_clamp)
            self.swarm.position = self._compute_position(self.swarm)
        # Obtain the final best_cost and the final best_position
        final_best_cost = self.swarm.best_cost.copy()
        final_best_pos = self.swarm.best_pos.copy()
        end_report(final_best_cost, final_best_pos, verbose, logger=self.logger)
        return (final_best_cost, final_best_pos)

    def _compute_position(self, swarm):
        """Updates the position matrix of the swarm.

        This computes the next position in a binary swarm. It compares the
        sigmoid output of the velocity-matrix and compares it with a randomly
        generated matrix.

        Parameters
        ----------
        swarm: pyswarms.backend.swarms.Swarm
            a Swarm class
        """
        return (np.random.random_sample(size=swarm.dimensions) < self._sigmoid
               (swarm.velocity)) * 1 

    def _sigmoid(self, x):
        """Helper method for the sigmoid function.

        Parameters
        ----------
        x : numpy.ndarray
            Input vector for sigmoid computation

        Returns
        -------
        numpy.ndarray
            Output sigmoid computation
        """
        return 1 / (1 + np.exp(x))




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import pytest
import numpy as np

# Import from package
from pyswarms.single import GlobalBestPSO
from pyswarms.utils.functions.single_obj import sphere_func

@pytest.mark.parametrize('options', [
    {'c2':0.7, 'w':0.5},
    {'c1':0.5, 'w':0.5},
    {'c1':0.5, 'c2':0.7}
])
def test_keyword_exception(options):
    """Tests if exceptions are thrown when keywords are missing"""
    with pytest.raises(KeyError):
        GlobalBestPSO(5, 2, options)

@pytest.mark.parametrize('bounds', [
    tuple(np.array([-5,-5])),
    (np.array([-5,-5,-5]), np.array([5,5])),
    (np.array([-5,-5,-5]), np.array([5,5,5]))
])
def test_bounds_size_exception(bounds, options):
    """Tests if exceptions are raised when bound sizes are wrong"""
    with pytest.raises(IndexError):
        GlobalBestPSO(5, 2, options=options, bounds=bounds)

@pytest.mark.parametrize('bounds', [
    (np.array([5,5]), np.array([-5,-5])),
    (np.array([5,-5]), np.array([-5,5]))
])
def test_bounds_maxmin_exception(bounds, options):
    """Tests if the max bounds is less than min bounds and vice-versa"""
    with pytest.raises(ValueError):
        GlobalBestPSO(5, 2, options=options, bounds=bounds)

@pytest.mark.parametrize('bounds',[
    [np.array([-5, -5]), np.array([5,5])],
    np.array([np.array([-5, -5]), np.array([5, 5])])
])
def test_bound_type_exception(bounds, options):
    """Tests if exception is raised when bound type is not a tuple"""
    with pytest.raises(TypeError):
        GlobalBestPSO(5,2, options=options, bounds=bounds)

@pytest.mark.parametrize('velocity_clamp', [(1,1,1), (2,3,1)])
def test_vclamp_shape_exception(velocity_clamp, options):
    """Tests if exception is raised when velocity_clamp's size is not equal
    to 2"""
    with pytest.raises(IndexError):
        GlobalBestPSO(5, 2, velocity_clamp=velocity_clamp, options=options)

@pytest.mark.parametrize('velocity_clamp', [(3,2),(10,8)])
def test_vclamp_maxmin_exception(velocity_clamp, options):
    """Tests if the max velocity_clamp is less than min velocity_clamp and
    vice-versa"""
    with pytest.raises(ValueError):
        GlobalBestPSO(5, 2, velocity_clamp=velocity_clamp, options=options)

@pytest.mark.parametrize('err, center',
    [(IndexError, [1.5, 3.2, 2.5])])
def test_center_exception(err, center, options):
    """Tests if exception is thrown when center is not a list or of different shape"""
    with pytest.raises(err):
        GlobalBestPSO(5, 2, center=center, options=options)

def test_reset_default_values(gbest_reset):
    """Tests if best cost and best pos are set properly when the reset()
    method is called"""
    assert gbest_reset.swarm.best_cost == np.inf
    assert set(gbest_reset.swarm.best_pos) == set(np.array([]))

def test_training_history_shape(gbest_history):
    """Test if training histories are of expected shape"""
    assert gbest_history.get_cost_history.shape == (1000,)
    assert gbest_history.get_mean_pbest_history.shape == (1000,)
    assert gbest_history.get_mean_neighbor_history.shape == (1000,)
    assert gbest_history.get_pos_history.shape == (1000, 10, 2)
    assert gbest_history.get_velocity_history.shape == (1000, 10, 2)

def test_ftol_effect(options):
    """Test if setting the ftol breaks the optimization process accodingly"""
    pso = GlobalBestPSO(10, 2, options=options, ftol=1e-1)
    pso.optimize(sphere_func, 2000, verbose=0)
    assert pso.get_cost_history.shape != (2000,)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import os
import pytest
import numpy as np
import matplotlib as mpl

if os.environ.get('DISPLAY','') == '':
    mpl.use('Agg')

from matplotlib.axes._subplots import SubplotBase
from matplotlib.animation import FuncAnimation

# Import from package
from pyswarms.utils.environments import PlotEnvironment
from pyswarms.utils.functions.single_obj import sphere_func

class_methods = [
    'get_cost_history',
    'get_pos_history',
    'get_velocity_history',
    'optimize',
    'reset'
]

@pytest.mark.parametrize('attributes', [i for i in enumerate(class_methods)])
def test_getters_pso(mock_pso, attributes):
    """Tests an instance of the PSO class and should raise an exception when the class has missing attributes"""
    idx, _ = attributes
    with pytest.raises(AttributeError):
            m = mock_pso(idx)
            PlotEnvironment(m, sphere_func, 100)

def test_plot_cost_return_type(plot_environment):
    """Tests if plot_cost() returns a SubplotBase instance"""
    assert isinstance(plot_environment.plot_cost(),SubplotBase)

def test_plot2D_return_type(plot_environment):
    """Test if plot_particles2D() returns a FuncAnimation instance"""
    assert isinstance(plot_environment.plot_particles2D(), FuncAnimation)

def test_plot3D_return_type(plot_environment):
    """Test if plot_particles3D() returns a FuncAnimation instance"""
    assert isinstance(plot_environment.plot_particles3D(), FuncAnimation)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import pytest
import numpy as np

# Import from package
import pyswarms.backend as P


@pytest.mark.parametrize('bounds', [None, ([2,2,2], [5,5,5]), ([-1,-1,0], [2,2,5])])
@pytest.mark.parametrize('center', [1, [3,3,3], [0.2,0.2,0.1]])
def test_generate_swarm_return_values(bounds, center):
    """Tests if generate_swarm() returns expected values"""
    pos = P.generate_swarm(n_particles=2, dimensions=3, bounds=bounds,
                           center=center)
    if bounds is None:
        min_bounds, max_bounds = (0.0, 1.00)
    else:
        min_bounds, max_bounds = bounds
    lower_bound = center * np.array(min_bounds)
    upper_bound = center * np.array(max_bounds)
    assert (pos <= upper_bound).all() and (pos >= lower_bound).all()

@pytest.mark.parametrize('bounds', [None, ([1,1,1], [10,10,10])])
@pytest.mark.parametrize('init_pos', [None, np.array([[2,5,6],[7,2,1]])])
def test_generate_swarm_bounds_init_pos(bounds, init_pos):
    """Tests if generate_swarm() returns expected values given init_pos and bounds"""
    pos = P.generate_swarm(n_particles=2, dimensions=3, bounds=bounds,
                           init_pos=init_pos)
    if (bounds is None) and (init_pos is None):
        min_bounds, max_bounds = (0.0, 1.00)
    elif (bounds is None) and (init_pos is not None):
        min_bounds, max_bounds = (-np.inf, np.inf)
    else:
        min_bounds, max_bounds = bounds
    lower_bound = np.array(min_bounds)
    upper_bound = np.array(max_bounds)
    assert (pos <= upper_bound).all() and (pos >= lower_bound).all()

def test_generate_swarm_out_of_bounds():
    """Tests if generate_swarm() raises ValueError when initialized with the wrong value"""
    bounds = ([1,1,1], [5,5,5])
    init_pos = np.array([[-2,3,3], [6,8,1]])
    with pytest.raises(ValueError):
        pos = P.generate_swarm(n_particles=2, dimensions=3, bounds=bounds,
                               init_pos=init_pos)

@pytest.mark.parametrize('binary', [False, True])
def test_generate_discrete_binary_swarm(binary):
    """Tests if generate_discrete_swarm(binary=True) returns expected values"""
    dims = 3
    pos = P.generate_discrete_swarm(n_particles=2, dimensions=dims, binary=binary)
    if binary:
        assert len(np.unique(pos)) == 2
    else:
        assert (np.max(pos, axis=1) == dims - 1).all()

@pytest.mark.parametrize('init_pos', [None, np.array([[4,2,1], [1,4,6]])])
def test_generate_discrete_swarm(init_pos):
    """Tests if init_pos actually sets the position properly"""
    dims = 3
    pos = P.generate_discrete_swarm(n_particles=2, dimensions=dims, init_pos=init_pos)
    if init_pos is None:
        assert (np.max(pos, axis=1) == dims - 1).all()
    else:
        assert np.equal(pos, init_pos).all()

@pytest.mark.parametrize('clamp', [None, (0,1), (2,5), (1,6)])
def test_generate_velocity_return_values(clamp):
    """Tests if generate_velocity() returns expected values"""
    min_clamp, max_clamp = (0,1) if clamp == None else clamp
    velocity = P.generate_velocity(n_particles=2, dimensions=3, clamp=clamp)
    assert (velocity <= max_clamp).all() and (velocity >= min_clamp).all()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import pytest
import numpy as np
from collections import namedtuple

# Import from package
from pyswarms.utils.functions import single_obj as fx

def test_sphere_output_size(common_minima, targetdim):
    """Tests sphere output size."""
    assert fx.sphere_func(common_minima).shape == targetdim

def test_rastrigin_output_size(common_minima, targetdim):
    """Tests rastrigin output size."""
    assert fx.rastrigin_func(common_minima).shape == targetdim

def test_ackley_output_size(common_minima, targetdim):
    """Tests ackley output size."""
    assert fx.ackley_func(common_minima).shape == targetdim

def test_rosenbrock_output_size(common_minima, targetdim):
    """Tests rosenbrock output size."""
    assert fx.rosenbrock_func(common_minima).shape == targetdim

def test_beale_output_size(common_minima, targetdim):
    """Tests beale output size."""
    assert fx.beale_func(common_minima).shape == targetdim

def test_goldstein_output_size(common_minima, targetdim):
    """Test goldstein output size."""
    assert fx.goldstein_func(common_minima).shape == targetdim

def test_booth_output_size(common_minima, targetdim):
    """Test booth output size."""
    assert fx.booth_func(common_minima).shape == targetdim

def test_bukin6_output_size(common_minima2, targetdim):
    """Test bukin6 output size."""
    assert fx.bukin6_func([-10,0] * common_minima2).shape == targetdim

def test_levi_output_size(common_minima, targetdim):
    """Test levi output size."""
    assert fx.levi_func(common_minima).shape == targetdim

def test_schaffer2_output_size(common_minima, targetdim):
    """Test schaffer2 output size."""
    assert fx.schaffer2_func(common_minima).shape == targetdim



# -*- coding: utf-8 -*-

"""
A Ring Network Topology

This class implements a star topology where all particles are connected in a
ring-like fashion. This social behavior is often found in LocalBest PSO
optimizers.
"""

# Import from stdlib
import logging

# Import modules
import numpy as np
from scipy.spatial import cKDTree

# Import from package
from .. import operators as ops
from .base import Topology

# Create a logger
logger = logging.getLogger(__name__)

class Ring(Topology):

    def __init__(self):
        super(Ring, self).__init__()

    def compute_gbest(self, swarm, p, k):
        """Updates the global best using a neighborhood approach

        This uses the cKDTree method from :code:`scipy` to obtain the nearest
        neighbours

        Parameters
        ----------
        swarm : pyswarms.backend.swarms.Swarm
            a Swarm instance
        k : int
            number of neighbors to be considered. Must be a
            positive integer less than :code:`n_particles`
        p: int {1,2}
            the Minkowski p-norm to use. 1 is the
            sum-of-absolute values (or L1 distance) while 2 is
            the Euclidean (or L2) distance.

        Returns
        -------
        numpy.ndarray
            Best position of shape :code:`(n_dimensions, )`
        float
            Best cost
        """
        try:
            # Obtain the nearest-neighbors for each particle
            tree = cKDTree(swarm.position)
            _, idx = tree.query(swarm.position, p=p, k=k)

            # Map the computed costs to the neighbour indices and take the
            # argmin. If k-neighbors is equal to 1, then the swarm acts
            # independently of each other.
            if k == 1:
                # The minimum index is itself, no mapping needed.
                best_neighbor = swarm.pbest_cost[idx][:, np.newaxis].argmin(axis=1)
            else:
                idx_min = swarm.pbest_cost[idx].argmin(axis=1)
                best_neighbor = idx[np.arange(len(idx)), idx_min]
            # Obtain best cost and position
            best_cost = np.min(swarm.pbest_cost[best_neighbor])
            best_pos = swarm.pbest_pos[np.argmin(swarm.pbest_cost[best_neighbor])]
        except AttributeError:
            msg = 'Please pass a Swarm class. You passed {}'.format(type(swarm))
            logger.error(msg)
            raise
        else:
            return (best_pos, best_cost)

    def compute_velocity(self, swarm, clamp=None):
        """Computes the velocity matrix

        This method updates the velocity matrix using the best and current
        positions of the swarm. The velocity matrix is computed using the
        cognitive and social terms of the swarm.
        
        A sample usage can be seen with the following:

        .. code-block :: python

            import pyswarms.backend as P
            from pyswarms.swarms.backend import Swarm
            from pyswarms.backend.topology import Star

            my_swarm = P.create_swarm(n_particles, dimensions)
            my_topology = Ring()

            for i in range(iters):
                # Inside the for-loop
                my_swarm.velocity = my_topology.update_velocity(my_swarm, clamp)

        Parameters
        ----------
        swarm : pyswarms.backend.swarms.Swarm
            a Swarm instance
        clamp : tuple of floats (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum velocity
            and the second entry is the maximum velocity. It
            sets the limits for velocity clamping.

        Returns
        -------
        numpy.ndarray
            Updated velocity matrix
        """
        return ops.compute_velocity(swarm, clamp)

    def compute_position(self, swarm, bounds=None):
        """Updates the position matrix

        This method updates the position matrix given the current position and
        the velocity. If bounded, it waives updating the position.

        Parameters
        ----------
        swarm : pyswarms.backend.swarms.Swarm
            a Swarm instance
        bounds : tuple of :code:`np.ndarray` or list (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum bound while
            the second entry is the maximum bound. Each array must be of shape
            :code:`(dimensions,)`.

        Returns
        -------
        numpy.ndarray
            New position-matrix
        """
        return ops.compute_position(swarm, bounds)


# -*- coding: utf-8 -*-

"""
Swarm Operation Backend

This module abstracts various operations in the swarm such as updating
the personal best, finding neighbors, etc. You can use these methods
to specify how the swarm will behave.
"""

# Import from stdlib
import logging

# Import modules
import numpy as np

# Create a logger
logger = logging.getLogger(__name__)

def compute_pbest(swarm):
    """Takes a swarm instance and updates the personal best scores
    
    You can use this method to update your personal best positions.

    .. code-block:: python

        import pyswarms.backend as P
        from pyswarms.backend.swarms import Swarm

        my_swarm = P.create_swarm(n_particles, dimensions)

        # Inside the for-loop...
        for i in range(iters):
            # It updates the swarm internally
            my_swarm.pbest_pos, my_swarm.pbest_cost = P.update_pbest(my_swarm)

    It updates your :code:`current_pbest` with the personal bests acquired by
    comparing the (1) cost of the current positions and the (2) personal
    bests your swarm has attained.
    
    If the cost of the current position is less than the cost of the personal
    best, then the current position replaces the previous personal best
    position.

    Parameters
    ----------
    swarm : pyswarms.backend.swarm.Swarm
        a Swarm instance

    Returns
    -------
    numpy.ndarray
        New personal best positions of shape :code:`(n_particles, n_dimensions)`
    numpy.ndarray
        New personal best costs of shape :code:`(n_particles,)`
    """
    try:
        # Infer dimensions from positions
        dimensions = swarm.dimensions
        # Create a 1-D and 2-D mask based from comparisons
        mask_cost = (swarm.current_cost < swarm.pbest_cost)
        mask_pos = np.repeat(mask_cost[:, np.newaxis], swarm.dimensions, axis=1)
        # Apply masks
        new_pbest_pos = np.where(~mask_pos, swarm.pbest_pos, swarm.position)
        new_pbest_cost = np.where(~mask_cost, swarm.pbest_cost, swarm.current_cost)
    except AttributeError:
        msg = 'Please pass a Swarm class. You passed {}'.format(type(swarm))
        logger.error(msg)
        raise
    else:
        return (new_pbest_pos, new_pbest_cost)

def compute_velocity(swarm, clamp):
    """Updates the velocity matrix

    This method updates the velocity matrix using the best and current
    positions of the swarm. The velocity matrix is computed using the
    cognitive and social terms of the swarm.
    
    A sample usage can be seen with the following:

    .. code-block :: python

        import pyswarms.backend as P
        from pyswarms.swarms.backend import Swarm

        my_swarm = P.create_swarm(n_particles, dimensions)

        for i in range(iters):
            # Inside the for-loop
            my_swarm.velocity = update_velocity(my_swarm, clamp)

    Parameters
    ----------
    swarm : pyswarms.backend.swarms.Swarm
        a Swarm instance
    clamp : tuple of floats (default is :code:`None`)
        a tuple of size 2 where the first entry is the minimum velocity
        and the second entry is the maximum velocity. It
        sets the limits for velocity clamping.

    Returns
    -------
    numpy.ndarray
        Updated velocity matrix
    """
    try:
        # Prepare parameters
        swarm_size = swarm.position.shape
        c1 = swarm.options['c1']
        c2 = swarm.options['c2']
        w = swarm.options['w']
        # Compute for cognitive and social terms
        cognitive = (c1 * np.random.uniform(0,1, swarm_size) * (swarm.pbest_pos - swarm.position))
        social = (c2 * np.random.uniform(0, 1, swarm_size) * (swarm.best_pos - swarm.position))
        # Compute temp velocity (subject to clamping if possible)
        temp_velocity = (w * swarm.velocity) + cognitive + social

        if clamp is None:
            updated_velocity = temp_velocity
        else:
            min_velocity, max_velocity = clamp
            mask = np.logical_and(temp_velocity >= min_velocity,
                                temp_velocity <= max_velocity)
            updated_velocity = np.where(~mask, swarm.velocity, temp_velocity)
    except AttributeError:
        msg = 'Please pass a Swarm class. You passed {}'.format(type(swarm))
        logger.error(msg)
        raise
    except KeyError:
        msg = 'Missing keyword in swarm.options'
        logger.error(msg)
        raise
    else:
        return updated_velocity

def compute_position(swarm, bounds):
    """Updates the position matrix

    This method updates the position matrix given the current position and
    the velocity. If bounded, it waives updating the position.

    Parameters
    ----------
    swarm : pyswarms.backend.swarms.Swarm
        a Swarm instance
    bounds : tuple of :code:`np.ndarray` or list (default is :code:`None`)
        a tuple of size 2 where the first entry is the minimum bound while
        the second entry is the maximum bound. Each array must be of shape
        :code:`(dimensions,)`.

    Returns
    -------
    numpy.ndarray
        New position-matrix
    """
    try:
        temp_position = swarm.position.copy()
        temp_position += swarm.velocity

        if bounds is not None:
            lb, ub = bounds
            min_bounds = np.repeat(np.array(lb)[np.newaxis, :], swarm.n_particles, axis=0)
            max_bounds = np.repeat(np.array(ub)[np.newaxis, :], swarm.n_particles, axis=0)
            mask = (np.all(min_bounds <= temp_position, axis=1)
                * np.all(temp_position <= max_bounds, axis=1))
            mask = np.repeat(mask[:, np.newaxis], swarm.dimensions, axis=1)
            temp_position = np.where(~mask, swarm.position, temp_position)
        position = temp_position
    except AttributeError:
        msg = 'Please pass a Swarm class. You passed {}'.format(type(swarm))
        logger.error(msg)
        raise
    else:
        return position



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Fixtures for tests"""

# Import modules
import pytest
import numpy as np

# Import from package
from pyswarms.single import (GlobalBestPSO, LocalBestPSO)
from pyswarms.discrete import BinaryPSO
from pyswarms.utils.functions.single_obj import sphere_func

@pytest.fixture(scope='module')
def gbest_history():
    """Returns a GlobalBestPSO instance run for 1000 iterations for checking
    history"""
    pso = GlobalBestPSO(10, 2, {'c1': 0.5, 'c2': 0.7, 'w': 0.5})
    pso.optimize(sphere_func, 1000, verbose=0)
    return pso

@pytest.fixture(scope='module')
def gbest_reset():
    """Returns a GlobalBestPSO instance that has been run and reset to check
    default value"""
    pso = GlobalBestPSO(10, 2, {'c1': 0.5, 'c2': 0.7, 'w': 0.5})
    pso.optimize(sphere_func, 10, verbose=0)
    pso.reset()
    return pso

@pytest.fixture(scope='module')
def lbest_history():
    """Returns a LocalBestPSO instance run for 1000 iterations for checking
    history"""
    pso = LocalBestPSO(10, 2, {'c1': 0.5, 'c2': 0.7, 'w': 0.5, 'k':2, 'p':2})
    pso.optimize(sphere_func, 1000, verbose=0)
    return pso

@pytest.fixture(scope='module')
def lbest_reset():
    """Returns a LocalBestPSO instance that has been run and reset to check
    default value"""
    pso = LocalBestPSO(10, 2, {'c1': 0.5, 'c2': 0.7, 'w': 0.5, 'k': 2, 'p': 2})
    pso.optimize(sphere_func, 10, verbose=0)
    pso.reset()
    return pso

@pytest.fixture(scope='module')
def binary_history():
    """Returns a BinaryPSO instance run for 1000 iterations for checking
    history"""
    pso = BinaryPSO(10, 2, {'c1': 0.5, 'c2': 0.7, 'w': 0.5, 'k':2, 'p':2})
    pso.optimize(sphere_func, 1000, verbose=0)
    return pso

@pytest.fixture(scope='module')
def binary_reset():
    """Returns a BinaryPSO instance that has been run and reset to check
    default value"""
    pso = BinaryPSO(10, 2, {'c1': 0.5, 'c2': 0.7, 'w': 0.5, 'k': 2, 'p': 2})
    pso.optimize(sphere_func, 10, verbose=0)
    pso.reset()
    return pso

@pytest.fixture
def options():
    """Default options dictionary for most PSO use-cases"""
    options_ = {'c1':0.5, 'c2':0.7, 'w':0.5, 'k':2, 'p':2}
    return options_


# -*- coding: utf-8 -*-

"""
Base class for Topologies

You can use this class to create your own topology. Note that every Topology
should implement a way to compute the (1) best particle, the (2) next
position, and the (3) next velocity given the Swarm's attributes at a given
timestep. Not implementing these methods will raise an error.

In addition, this class must interface with any class found in the
:mod:`pyswarms.backend.swarms.Swarm` module.
"""

class Topology(object):

    def __init__(self, **kwargs):
        """Initializes the class"""
        pass

    def compute_gbest(self, swarm):
        """Computes the best particle of the swarm and returns the cost and
        position"""
        raise NotImplementedError("Topology::compute_gbest()")

    def compute_position(self, swarm):
        """Updates the swarm's position-matrix"""
        raise NotImplementedError("Topology::compute_position()")

    def compute_velocity(self, swarm):
        """Updates the swarm's velocity-matrix"""
        raise NotImplementedError("Topology::compute_velocity()")


# -*- coding: utf-8 -*-

"""
A Star Network Topology

This class implements a star topology where all particles are connected to
one another. This social behavior is often found in GlobalBest PSO
optimizers.
"""

# Import from stdlib
import logging

# Import modules
import numpy as np

# Import from package
from .. import operators as ops
from .base import Topology

# Create a logger
logger = logging.getLogger(__name__)

class Star(Topology):

    def __init__(self):
        super(Star, self).__init__()

    def compute_gbest(self, swarm):
        """Obtains the global best cost and position based on a star topology

        This method takes the current pbest_pos and pbest_cost, then returns
        the minimum cost and position from the matrix. It should be used in
        tandem with an if statement

        .. code-block:: python

            import pyswarms.backend as P
            from pyswarms.backend.swarms import Swarm
            from pyswarm.backend.topology import Star

            my_swarm = P.create_swarm(n_particles, dimensions)
            my_topology = Star()

            # If the minima of the pbest_cost is less than the best_cost
            if np.min(pbest_cost) < best_cost:
                # Update best_cost and position
                swarm.best_pos, swarm.best_cost = my_topology.compute_best_particle(my_swarm)

        Parameters
        ----------
        swarm : pyswarms.backend.swarm.Swarm
            a Swarm instance

        Returns
        -------
        numpy.ndarray
            Best position of shape :code:`(n_dimensions, )`
        float
            Best cost
        """
        try:
            best_pos = swarm.pbest_pos[np.argmin(swarm.pbest_cost)]
            best_cost = np.min(swarm.pbest_cost)
        except AttributeError:
            msg = 'Please pass a Swarm class. You passed {}'.format(type(swarm))
            logger.error(msg)
            raise
        else:
            return (best_pos, best_cost)

    def compute_velocity(self, swarm, clamp=None):
        """Computes the velocity matrix

        This method updates the velocity matrix using the best and current
        positions of the swarm. The velocity matrix is computed using the
        cognitive and social terms of the swarm.
        
        A sample usage can be seen with the following:

        .. code-block :: python

            import pyswarms.backend as P
            from pyswarms.swarms.backend import Swarm
            from pyswarms.backend.topology import Star

            my_swarm = P.create_swarm(n_particles, dimensions)
            my_topology = Star()

            for i in range(iters):
                # Inside the for-loop
                my_swarm.velocity = my_topology.update_velocity(my_swarm, clamp)

        Parameters
        ----------
        swarm : pyswarms.backend.swarms.Swarm
            a Swarm instance
        clamp : tuple of floats (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum velocity
            and the second entry is the maximum velocity. It
            sets the limits for velocity clamping.

        Returns
        -------
        numpy.ndarray
            Updated velocity matrix
        """
        return ops.compute_velocity(swarm, clamp)

    def compute_position(self, swarm, bounds=None):
        """Updates the position matrix

        This method updates the position matrix given the current position and
        the velocity. If bounded, it waives updating the position.

        Parameters
        ----------
        swarm : pyswarms.backend.swarms.Swarm
            a Swarm instance
        bounds : tuple of :code:`np.ndarray` or list (default is :code:`None`)
            a tuple of size 2 where the first entry is the minimum bound while
            the second entry is the maximum bound. Each array must be of shape
            :code:`(dimensions,)`.

        Returns
        -------
        numpy.ndarray
            New position-matrix
        """
        return ops.compute_position(swarm, bounds)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'PyYAML==3.12',
    'future==0.16.0',
    'scipy>=0.17.0',
    'numpy>=1.13.0',
    'matplotlib>=1.3.1',
    'mock==2.0.0',
    'pytest==3.2.1',
    'attrs==18.1.0'
]

setup_requirements = [
    # TODO(ljvmiranda921): put setup requirements (distutils extensions, etc.) here
]

test_requirements = [
    'PyYAML==3.12',
    'future==0.16.0',
    'scipy>=0.17.0',
    'numpy>=1.13.0',
    'matplotlib>=1.3.1',
    'mock==2.0.0',
    'pytest==3.2.1',
    'attrs==18.1.0'
]

setup(
    name='pyswarms',
    version='0.2.0',
    description="A Python-based Particle Swarm Optimization (PSO) library.",
    long_description=readme + '\n\n' + history,
    author="Lester James V. Miranda",
    author_email='ljvmiranda@gmail.com',
    url='https://github.com/ljvmiranda921/pyswarms',
    packages=find_packages(exclude=['docs', 'tests']),
    include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords='pyswarms',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Topic :: Scientific/Engineering',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    setup_requires=setup_requirements,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Fixtures for tests"""

# Import modules
import pytest
import numpy as np

# Import from package
from pyswarms.utils.search.grid_search import GridSearch
from pyswarms.utils.search.random_search import RandomSearch
from pyswarms.single import LocalBestPSO
from pyswarms.utils.functions.single_obj import sphere_func

@pytest.fixture
def grid():
    """Returns a GridSearch instance"""
    options = {'c1': [1, 2, 3],
               'c2': [1, 2, 3],
               'k' : [5, 10, 15],
               'w' : [0.9, 0.7, 0.4],
               'p' : [1]}
    return GridSearch(LocalBestPSO, n_particles=40,
        dimensions=20, options=options, objective_func=sphere_func, iters=10, bounds=None)

@pytest.fixture
def grid_mini():
    """Returns a GridSearch instance with a smaller search-space"""
    options = {'c1': [1, 2],
               'c2': 6,
               'k' : 5,
               'w' : 0.9,
               'p' : 0}
    return GridSearch(LocalBestPSO, n_particles=40,
        dimensions=20, options=options, objective_func=sphere_func, iters=10, bounds=None)

@pytest.fixture
def random_unbounded():
    """Returns a RandomSearch instance without bounds"""
    options = {'c1': [1, 5],
               'c2': [6, 10],
               'k' : [11, 15],
               'w' : [0.4, 0.9],
               'p' : 1}
    return RandomSearch(LocalBestPSO, n_particles=40, dimensions=20, options=options, objective_func=sphere_func, iters=10, n_selection_iters=100, bounds=None)

@pytest.fixture
def random_bounded():
    """Returns a RandomSearch instance with bounds"""
    bounds = (np.array([-5,-5]), np.array([5,5]))
    options = {'c1': [1, 5],
               'c2': [6, 10],
               'k' : [11, 15],
               'w' : [0.4, 0.9],
               'p' : 1}
    return RandomSearch(LocalBestPSO, n_particles=40, dimensions=20, options=options, objective_func=sphere_func, iters=10, n_selection_iters=100, bounds=bounds)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `pyswarms` package."""

# Import modules
import pytest
import numpy as np
from collections import namedtuple

# Import from package
from pyswarms.utils.functions import single_obj as fx

Bounds = namedtuple('Bounds', 'low high')
b = {
    # Define all bounds here
    'rastrigin' : Bounds(low=-5.12, high=5.12),
    'ackley'    : Bounds(low=-32, high=32),
    'beale'     : Bounds(low=-4.5, high=4.5),
    'goldstein' : Bounds(low=-2, high=-2),
    'booth'     : Bounds(low=-10, high=10),
    'matyas'    : Bounds(low=-10, high=10),
    'levi'      : Bounds(low=-10, high=10),
    'schaffer2' : Bounds(low=-100, high=100)

}

def test_rastrigin_bound_fail(outbound):
    """Test rastrigin bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b['rastrigin'].low, b['rastrigin'].high, size=(3,2))
        fx.rastrigin_func(x)

def test_ackley_bound_fail(outbound):
    """Test ackley bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b['ackley'].low, b['ackley'].high, size=(3,2))
        fx.ackley_func(x)

def test_beale_bound_fail(outbound):
    """Test beale bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b['beale'].low, b['beale'].high, size=(3,2))
        fx.beale_func(x)

def test_goldstein_bound_fail(outbound):
    """Test goldstein bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b['goldstein'].low, b['goldstein'].high, size=(3,2))
        fx.goldstein_func(x)

def test_booth_bound_fail(outbound):
    """Test booth bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b['booth'].low, b['booth'].high, size=(3,2))
        fx.booth_func(x)

@pytest.mark.parametrize("x", [
    -np.random.uniform(15.001, 100, (3,2)),
    np.random.uniform(-5.001, -3.001, (3,2)),
    np.random.uniform(-3.001, -100, (3,2))
])
def test_bukin6_bound_fail(x):
    """Test bukin6 bound exception"""
    with pytest.raises(ValueError):
        fx.bukin6_func(x)

def test_matyas_bound_fail(outbound):
    """Test matyas bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b['matyas'].low, b['matyas'].high, size=(3,2))
        fx.matyas_func(x)

def test_levi_bound_fail(outbound):
    """Test levi bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b['levi'].low, b['levi'].high, size=(3,2))
        fx.levi_func(x)

def test_schaffer2_bound_fail(outbound):
    """Test schaffer2 bound exception"""
    with pytest.raises(ValueError):
        x = outbound(b['schaffer2'].low, b['schaffer2'].high, tol=200, size=(3,2))
        fx.schaffer2_func(x)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Fixtures for tests"""

# Import modules
import pytest
import numpy as np

# Import from package
from pyswarms.backend.swarms import Swarm

@pytest.fixture
def swarm():
    """A contrived instance of the Swarm class at a certain timestep"""
    attrs_at_t = {
        'position' : np.array([[5,5,5], [3,3,3], [1,1,1]]),
        'velocity' : np.array([[1,1,1], [1,1,1], [1,1,1]]),
        'current_cost' : np.array([2,2,2]),
        'pbest_cost' : np.array([1,2,3]),
        'pbest_pos' : np.array([[1,2,3], [4,5,6], [7,8,9]]),
        'best_cost' : 1,
        'best_pos' : np.array([1,1,1]),
        'options' : {'c1' : 0.5, 'c2': 1, 'w': 2}
    }
    return Swarm(**attrs_at_t)




"""
The :mod:`pyswarms.single` module implements various techniques in
continuous single-objective optimization. These require only one
objective function that can be optimized in a continuous space.
"""

from .global_best import GlobalBestPSO
from .local_best import LocalBestPSO

__all__ = [
    "GlobalBestPSO",
    "LocalBestPSO"
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import modules
import pytest

@pytest.mark.parametrize('maximum', [True, False])
def test_search_best_options_return_type(grid, maximum):
    """Tests if best options returns a dictionary"""
    _, best_options = grid.search(maximum)
    assert isinstance(best_options, dict)

def test_grid_output(grid_mini):
    """Tests if generate_grid function returns expected value"""
    expected = [{'c1': 1, 'c2': 6, 'k': 5, 'w': 0.9, 'p': 0},
        {'c1': 2, 'c2': 6, 'k': 5, 'w': 0.9, 'p': 0}]
    assert grid_mini.generate_grid() == expected 